/*
 ***********************************************************************************************************************
 *
 * Copyright (c) Infineon Technologies AG
 * All rights reserved.
 *
 * The applicable license agreement can be found at this pack's installation directory in the file
 * license/IFX_SW_Licence_MOTIX_LITIX.txt
 *
 **********************************************************************************************************************/

/*******************************************************************************
**                             Author(s) Identity                             **
********************************************************************************
** Initials     Name                                                          **
** ---------------------------------------------------------------------------**
** ISLE         ISLE Ilmenau                                                  **
** BG           Blandine Guillot                                              **
** JO           Julia Ott                                                     **
*******************************************************************************/

/*******************************************************************************
**                          Revision Control History                          **
********************************************************************************
** V1.0.0: 2016-07-20, ISLE: Initial version of revision history              **
** V1.0.1: 2020-04-15, BG:   Updated revision history                         **
** V1.0.2: 2020-07-21, BG:   EP-439: Formatted .h/.c files                    **
** V1.1.0: 2022-07-07, JO:   EP-1206: Updated LIN example to the following    **
**                           signal: min = 0, max = 5000, scale = 1,          **
**                           offset = -2500                                   **
** V1.2.0: 2024-11-06, JO:   EP-1535: Set C version to C99                    **
**                           Set compiler warning level to 'all warnings'     **
** V1.2.1: 2025-01-02, JO:   EP-1493: Updated license                         **
*******************************************************************************/

/*******************************************************************************
**                                  Abstract                                  **
********************************************************************************
** BLDC BEMF + LIN: Motor Drive with sensorless block commutation (BEMF)      **
********************************************************************************
** This example is configured to work with the Trinamic QBL 4208-61-04-013    **
** motor.                                                                     **
** The speed is sent through LIN communication - it is suggested to send      **
** speed commands with a separate board running the LIN Master example.       **
** Turn the Poti to make the motor run (both negative and positive directions **
** are possible).                                                             **
*******************************************************************************/

#include "tle_device.h"
#include "Emo_BEMF.h"
#include "linmain.h"
#include "genLinConfig.h"
#include "lin_driver_api.h"

static void Main_lStartMotor(void);
static void Main_lStopMotor(void);


int main(void)
{
  static uint16 speed;
  static sint16 si16_speed;
  /* Initialization of hardware modules based on Config Wizard configuration */
  TLE_Init();
  /* Initialize E-Motor application */
  Emo_Init();
  /* Initialize IHR LIN Low Level Driver */
  LIN_LLD_Init();

  for (;;)
  {
    /* Service watchdog */
    (void)WDT1_Service();
    LIN_LLD_Service();

    /* Check if a new LIN frame was received */
    if (l_flg_tst_RefSpeed())
    {
      /* Get the RefSpeed sent via LIN */
      speed = l_u16_rd_RefSpeed();
      si16_speed = (sint16)speed - (sint16)2500;

      /* Check if speed is above 300rpm */
      if ((si16_speed > 300) || (si16_speed < -300))
      {
        /* Set speed as reference speed for the BLDC motor */
        if ((si16_speed < 3000) || (si16_speed > -3000))
        {
          Emo_SetRefSpeed(si16_speed);
        }

        /* Clear existing BDRV interrupt status */
        SCUPM->BDRV_ISCLR.reg = (uint32)0xFFFFFFFF;
        /* Start the motor if not running yet */
        Main_lStartMotor();
      }
      else
      {
        /* If speed is below 300rpm, stop the motor */
        Main_lStopMotor();
        /* Reinitialize motor control parameters */
        Emo_Init();
      }

      /* Send current motor speed back to LIN master */
      l_u16_wr_ActSpeed(Emo_GetSpeed());
    }
  }
}


void Main_HandleSysTick(void)
{
}


static void Main_lStartMotor(void)
{
  uint32 Error;
  Error = EMO_ERROR_NONE;

  if (Error == EMO_ERROR_NONE)
  {
    Error = Emo_StartMotor();
  }
}


static void Main_lStopMotor(void)
{
  uint32 Error;
  Error = Emo_StopMotor();

  if (Error != EMO_ERROR_NONE)
  {
  }
}

