/*
 ***********************************************************************************************************************
 *
 * Copyright (c) Infineon Technologies AG
 * All rights reserved.
 *
 * The applicable license agreement can be found at this pack's installation directory in the file
 * license/IFX_SW_Licence_MOTIX_LITIX.txt
 *
 **********************************************************************************************************************/

/*******************************************************************************
**                             Author(s) Identity                             **
********************************************************************************
** Initials     Name                                                          **
** ---------------------------------------------------------------------------**
** BG           Blandine Guillot                                              **
** JO           Julia Ott                                                     **
** CK           Chander Kumar                                                 **
*******************************************************************************/

/*******************************************************************************
**                          Revision Control History                          **
********************************************************************************
** V1.0.0: 2020-04-15, BG:   Initial version of revision history              **
** V1.0.1: 2020-07-21, BG:   EP-439: Formatted .h/.c files                    **
** V1.1.0: 2022-07-07, JO:   EP-1206: Updated LIN example to the following    **
**                           signal: min = 0, max = 5000, scale = 1,          **
**                           offset = -2500                                   **
** V1.1.1: 2022-07-18, CK:   EP-1147: Added selection of motor in CW and      **
**                           modified hall patterns in EmoCcu_Cfg.c           **
** V1.2.0: 2024-11-06, JO:   EP-1535: Set C version to C99                    **
**                           Set compiler warning level to 'all warnings'     **
** V1.2.1: 2025-01-02, JO:   EP-1493: Updated license                         **
*******************************************************************************/

/*******************************************************************************
**                                  Abstract                                  **
********************************************************************************
** BLDC: Motor Drive with block commutation and Hall sensor                   **
********************************************************************************
** The motor speed is sent through LIN. It is suggested to send speed         **
** commands with a separate board, running with the "LIN Master" example.     **
**                                                                            **
** Motor connection, QBL4208:                                                 **
** Phase 1             : yellow                                               **
** Phase 2             : red                                                  **
** Phase 3             : black                                                **
** Hall A - P0.3       : blue  (Motor Hall A)                                 **
** Hall B - P0.4       : green (Motor Hall B)                                 **
** Hall C - P1.2       : white (Motor Hall C)                                 **
** Hall Supply - VDDEXT: red                                                  **
** Hall Gnd - GND      : black                                                **
**                                                                            **
** Motor connection, DB42S03 Nanotec:                                         **
** Phase 1             : yellow                                               **
** Phase 2             : red                                                  **
** Phase 3             : black                                                **
** Hall A - P0.3       : blue  (Motor Hall A)                                 **
** Hall B - P0.4       : white (Motor Hall B)                                 **
** Hall C - P1.2       : green (Motor Hall C)                                 **
** Hall Supply - VDDEXT: red                                                  **
** Hall Gnd - GND      : black                                                **
*******************************************************************************/

#include "tle_device.h"
#include "Emo.h"
#include "linmain.h"
#include "genLinConfig.h"
#include "lin_driver_api.h"

static void Main_lStartMotor(void);
static void Main_lStopMotor(void);


int main(void)
{
  static sint16 speed;
  static sint16 si16_speed;
  /* Initialization of hardware modules based on Config Wizard configuration */
  TLE_Init();
  /* Initialize E-Motor application */
  Emo_Init();
  /* Initialize IHR LIN Low Level Driver */
  LIN_LLD_Init();

  for (;;)
  {
    /* Service watchdog */
    (void)WDT1_Service();
    LIN_LLD_Service();

    /* Check if a new LIN frame was received */
    if (l_flg_tst_RefSpeed())
    {
      /* Get the RefSpeed sent via LIN */
      speed = l_u16_rd_RefSpeed();
      si16_speed = (sint16)speed - (sint16)2500;

      /* Check if speed is above 300rpm */
      if ((si16_speed > 300) || (si16_speed < -300))
      {
        /* Set speed as reference speed for the BLDC motor */
        if ((si16_speed < 3000) || (si16_speed > -3000))
        {
          Emo_SetRefSpeed(si16_speed);
        }
        
        /* Start the motor if not running yet */
        Main_lStartMotor();
      }
      else
      {
        /* If speed is below 300rpm, stop the motor */
        Main_lStopMotor();
        /* Reinitialise motor control parameters */
        Emo_Init();
      }
    }
  }
}


void Main_HandleSysTick(void)
{
  /* Callback function executed every ms for speed control */
  Emo_CtrlSpeed();
}


static void Main_lStartMotor(void)
{
  uint32 Error;
  Error = EMO_ERROR_NONE;

  if (Error == EMO_ERROR_NONE)
  {
    /* Start the motor with desired positive or negative speed in [rpm] */
    Error = Emo_StartMotor();
  }

  if (Error == EMO_ERROR_NONE)
  {
  }
  else /* Error */
  {
  }
}


static void Main_lStopMotor(void)
{
  uint32 Error;
  Error = Emo_StopMotor();

  if (Error != EMO_ERROR_NONE)
  {
  }
}

