/*
 ***********************************************************************************************************************
 *
 * Copyright (c) Infineon Technologies AG
 * All rights reserved.
 *
 * The applicable license agreement can be found at this pack's installation directory in the file
 * license/IFX_SW_Licence_MOTIX_LITIX.txt
 *
 **********************************************************************************************************************/
 
/*******************************************************************************
**                             Author(s) Identity                             **
********************************************************************************
** Initials     Name                                                          **
** ---------------------------------------------------------------------------**
** MK           Martin Kilian                                                 **
** JO           Julia Ott                                                     **
*******************************************************************************/

/*******************************************************************************
**                          Revision Control History                          **
********************************************************************************
** V1.0.0: 2021-05-14, MK, JO: Initial version of the example (C code and     **
**                             Config Wizard GUI)                             **
** V1.0.1: 2021-10-07, MK:     EP-974: bldc_motor_lib update                  **
**                             (V1.0.0 --> V1.0.1)                            **
** V1.0.2: 2024-11-06, JO:     EP-1535: Formatted file                        **
** V1.0.3: 2025-01-02, JO:     EP-1493: Updated license                       **
*******************************************************************************/

/*******************************************************************************
**                                  Abstract                                  **
********************************************************************************
**                            MCTRL Example: Hall                             **
********************************************************************************
** This example turns a motor with Hall sensors.                              **
** There are two options:                                                     **
** 1. Hall pattern known: In Config Wizard, select the option "Hall pattern   **
**    known" and enter the Hall Pattern. Connect the phases and Hall signals  **
**    accordingly.                                                            **
** 2. Hall pattern not known: The example offers the option to detect the     **
**    Hall pattern. In Config Wizard, select the option "Hall pattern not     **
**    known - to be measured" to enable the Hall pattern measurement.         **
**    You can let the example measure the Hall Pattern and send them to PC.   **
**    The Hall pattern are sent to the PC via stdio, 115.2kBaud on MON button.**
**    Then the Hall Pattern are known and you can then enter them in Config   **
**    Wizard.                                                                 **
**                                                                            **
** By default, the option "Hall Pattern not known - to be measured" is        **
** selected.                                                                  **
**                                                                            **
** Note: In case the Motor doesn't run smoothly, it might be that the Hall    **
** Pattern were not detected correctly. In this case, please reset the        **
** device and try again.                                                      **
**                                                                            **
** The potentiometer controls the motor, both directions are supported.       **
*******************************************************************************/

/*******************************************************************************
**                                  Includes                                  **
*******************************************************************************/
#include "tle_device.h"
#include "bldc_motor_lib.h"
#include "speedcontrol.h"
#include <stdlib.h>
#include <stdio.h>

/*******************************************************************************
**                          Main Definitions                                  **
*******************************************************************************/
#define FACTOR_POTI 3  /* 5000mV*FACTOR_POTI = rpm */
#define FACTOR_VOLT 10 /* Max voltage for Motor dutycycle */

/*******************************************************************************
**                          ISR Function Definitions                          **
*******************************************************************************/
void systick_1ms(void);

/*******************************************************************************
**                       Private Function Declarations                        **
*******************************************************************************/
void Main_Motor_Handling(void);
void printHallPtn(void);
void MON_buttonPressed(void);

/*******************************************************************************
**                        Static variable Definitions                         **
*******************************************************************************/
static uint16 poti_value_mV = 0u;
static bool b_sendData = false;
/*******************************************************************************
**                               Main function                                **
*******************************************************************************/
int main(void) 
{
  TLE_Init();                        /* Initialize TLE987x as selected in Config Wizard 2 */
  Motor_State_Set(MOTOR_STATE_STOP); /* Initialize motor state */
  Speed_Control_Init();              /* Initialize speedcontroller */
  
  for (;;)
  {    
    WDT1_Service(); 
    
    if (b_sendData == true)
    {
      printHallPtn();
      b_sendData = false;
    }
  } 
}

/*******************************************************************************
**                        Private Function Definitions                        **
*******************************************************************************/
/* Systick Interrupt for Speedcontrol */
void systick_1ms(void) 
{
  /* Main_Motor_Handling runs in the 1ms Systick Interrupt */
  Main_Motor_Handling(); 
}

/* Speedcontrol Routine 1ms*/
void Main_Motor_Handling(void)
{
  sint16 s16_poti_value_scaled;
  uint32 u32_motor_dir;
  
  ADC1_GetChResult_mV(&poti_value_mV, ADC1_P24); /* get poti value 0..5000mV */
  
  /* Limiter --> 0...5000 */
  if(poti_value_mV > ADC1_VREF_5000mV)
  {
    poti_value_mV = ADC1_VREF_5000mV;
  }
  
  /* Scale --> -2500...2500 */
  s16_poti_value_scaled = (sint16)poti_value_mV - (sint16)(ADC1_VREF_5000mV/2);
  
  /* Define direction (-2500...0: DIR_BACKWARD, 0...2500: DIR_FORWARD) */
  if (s16_poti_value_scaled < 0)
  {
    u32_motor_dir = DIR_BACKWARD;
  }
  else
  {
    u32_motor_dir = DIR_FORWARD;
  }
  
  if((s16_poti_value_scaled < -200) || (s16_poti_value_scaled > 200)) /* start motor at potentiometer values > 400mV */
  {
    switch(Motor_State_Get())
    {
      case MOTOR_STATE_STOP:     Motor_Init(u32_motor_dir);
                                 Speed_Control_Init();                           
                                 Motor_Startup();
                                 break;
      case MOTOR_STATE_RUNNING : 
                                #if (SC_USE_SC == 1)
                                  Motor_Direction_Set(u32_motor_dir);
                                  SC_Actspeed_Set((int)Motor_Speed_Get());                   /* Get speed from motor API and set speed control */
                                  SC_Refspeed_Set(abs(s16_poti_value_scaled)*FACTOR_POTI);   /* Set speed for speed control                    */
                                  Motor_VoltageDC_Set((uint32)Speed_Control()*FACTOR_VOLT);  /* Update Dutycycle by speed control output       */
                                #else
                                  Motor_Direction_Set(u32_motor_dir);
                                  Motor_Dutycycle_Set((uint32)abs(s16_poti_value_scaled)/3); /* Set Dutycycle 0% to 100.0% */
                                #endif /* SC_USE_SC */
                                 break;
      default                  : break;
    }
  }
  else /* stop motor at potentiometer values <= 400mV */
  {
      Motor_Stop();
  }
}

/* MON Falling Edge interrupt = Mon button pressed */
/* This interrupt has priority 15 /lowest) so it doesn't disturb motor control */
void MON_buttonPressed(void)
{
  /* Wait until MON button is released */
  MON_Debounce(1);

  b_sendData = true;
}

/* Function to print the Hall pattern via UART stdio */
void printHallPtn(void)
{
  uint8 u8_hall_msk;
  
  printf("\n");
  printf("Hall Pattern:\n");
  u8_hall_msk = 1;
  
  for(uint8 hall=0; hall<3; hall++)
  {
    printf("Hall %d:            ", hall);
    
    for(uint8 ptn=0; ptn<6; ptn++)
    {
      printf("%d     ", (Hall.patterns[ptn] & u8_hall_msk) >> hall);
    }
    printf("\n");
    
    u8_hall_msk = u8_hall_msk << 1u;
  }
  printf("Resulting Pattern: ");
  for(uint8 ptn=0; ptn<6; ptn++)
  {
    printf("0x%02X  ", Hall.patterns[ptn]);
  }
}

