/*
 ***********************************************************************************************************************
 *
 * Copyright (c) Infineon Technologies AG
 * All rights reserved.
 *
 * The applicable license agreement can be found at this pack's installation directory in the file
 * license/IFX_SW_Licence_MOTIX_LITIX.txt
 *
 **********************************************************************************************************************/

/*******************************************************************************
**                             Author(s) Identity                             **
********************************************************************************
** Initials     Name                                                          **
** ---------------------------------------------------------------------------**
** KC           Kay Claussen                                                  **
** BG           Blandine Guillot                                              **
** JO           Julia Ott                                                     **
*******************************************************************************/

/*******************************************************************************
**                          Revision Control History                          **
********************************************************************************
** V1.0.0: 2016-10-13, KC:   Initial version                                  **
** V1.1.0: 2020-04-15, BG:   Updated revision history                         **
** V1.1.1: 2020-07-21, BG:   EP-439: Formatted .h/.c files                    **
** V1.2.0: 2024-11-06, JO:   EP-1535: Set C version to C99                    **
** V1.2.1: 2025-01-02, JO:   EP-1493: Updated license                         **
*******************************************************************************/

/*******************************************************************************
**                                  Abstract                                  **
********************************************************************************
** LIN: Autobaudrate detection                                                **
********************************************************************************
** Timer 2 is configured to capture the LIN baudrate. Read out of baudrate    **
** is done after synch field detection in LIN_End_Of_Synch_Handler().         **
** Device generates interrupt on reception of                                 **
** - LIN Synch Field -> Toggles P0.1                                          **
** - LIN Synch Field errors -> Toggles P0.2                                   **
** - T2 Interrupt on falling edge of LIN signal-> Toggles P0.3                **
** - UART byte reception -> Toggles P1.0                                      **
** Variable LIN_baudrate_ticks keeps the captured time of 8 bits:             **
** LIN_baudrate_ticks is ~4110hex for 19200 Baud @ fsys = 40 MHz.             **
** 4110hex = 16656 x 25 ns = 416400 ns for 8 bits;                            **
** 416400 ns / 8 = 52.05 us = 19212 Baud                                      **
*******************************************************************************/

#include "tle_device.h"

#define ERRSYNC_EOFSYNC_BRK 0x38 /* mask for lin break, synch and synch error */

static uint16 LIN_baudrate_ticks;

int main(void)
{
  /* Initialization of hardware modules based on Config Wizard configuration */
  TLE_Init();

  for (;;)
  {
    (void)WDT1_Service();
  }
}

/* ISR for LIN End of Synch Field detection */
void LIN_End_Of_Synch_Handler(void)
{
  /* Toggle P0.1 */
  PORT_P01_Output_Toggle_Set();

  /* If break field AND synch field AND no sync error appeared then ... */
  if ((SCU->LINST.reg & ERRSYNC_EOFSYNC_BRK) == 0x18)
  {
    /* ... capture timer value */
    LIN_baudrate_ticks = TIMER2_Get_Capture();
  }
}

/* ISR for LIN synch filed error detection */
void LIN_Synch_Error_Handler(void)
{
  /* Toggle P0.2 */
  PORT_P02_Output_Toggle_Set();
}

/* ISR for T2 */
void TIMER2_Handler(void)
{
  /* Toggle P0.3 */
  PORT_P03_Output_Toggle_Set();
  TIMER2_Stop();
  TIMER2_Clear_Count();
  /* Clear  LIN field indicators */
  SCU->LINSCLR.reg = ERRSYNC_EOFSYNC_BRK;
  /* Disable external T2 interrupt */
  TIMER2_External_Int_Dis();
}

/* ISR for UART Receive */
void UART_Receive_Handler(void)
{
  /* Toggle P1.0 */
  PORT_P10_Output_Toggle_Set();
  /* In a LIN driver this part of the code should be located after reception of the last LIN byte or after the occurrence of an error on LIN. */
  /* This will reduce the number of T2 interrupts. */
  /* Clear pending T2 interrupt */
  TIMER2->T2ICLR.reg = 0x40;
  /* Enable external T2 interrupt */
  TIMER2_External_Int_En();
}
